package com.redhat.training.claims;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.redhat.training.claims.model.Claim;
import com.redhat.training.claims.model.ClaimClarificationRequests;
import com.redhat.training.claims.model.ClaimProcessingAborted;
import com.redhat.training.claims.model.ClaimResults;
import com.redhat.training.claims.model.Claimant;
import com.redhat.training.claims.model.Gender;

import org.kie.api.KieServices;
import org.kie.api.command.BatchExecutionCommand;
import org.kie.api.command.Command;
import org.kie.api.command.KieCommands;
import org.kie.api.runtime.ExecutionResults;
import org.kie.server.api.marshalling.MarshallingFormat;
import org.kie.server.api.model.ServiceResponse;
import org.kie.server.client.CredentialsProvider;
import org.kie.server.client.KieServicesClient;
import org.kie.server.client.KieServicesConfiguration;
import org.kie.server.client.KieServicesFactory;
import org.kie.server.client.RuleServicesClient;
import org.kie.server.client.credentials.EnteredCredentialsProvider;

public class SubmitClaimRemoteKie {
    private static final String KIE_SERVER_URL = "http://localhost:8080/kie-server/services/rest/server";
    private static final String USERNAME = "adminUser";
    private static final String PASSWORD = "test1234!";
    private static final String CONTAINER = "claims";

    public static void main(String... args) {
        CredentialsProvider credentialsProvider = new EnteredCredentialsProvider(USERNAME, PASSWORD);
        KieServicesConfiguration kieServicesConfig = KieServicesFactory.newRestConfiguration(KIE_SERVER_URL, credentialsProvider);
        kieServicesConfig.setMarshallingFormat(MarshallingFormat.JSON);

        KieServicesClient kieServicesClient = KieServicesFactory.newKieServicesClient(kieServicesConfig);
        RuleServicesClient client = kieServicesClient.getServicesClient(RuleServicesClient.class);

        Claimant c = new Claimant();
        c.setName("Tom Jones");
        c.setGender(Gender.MALE);

        Claim cl = new Claim();
        cl.setId(234);
        cl.setAmount(500.00);
        cl.setReason("Test 123");
        cl.getInvolvedPersons().add(c);

        c.getClaims().add(cl);

        ClaimResults cr = new ClaimResults(cl.getId());
        ClaimClarificationRequests ccr = new ClaimClarificationRequests(cl.getId());
        ClaimProcessingAborted cpa = new ClaimProcessingAborted(cl.getId());

        HashMap<String, Object> params = new HashMap<>();
        params.put("claimant", c);
        params.put("claim", cl);
        params.put("claimResults", cr);
        params.put("claimClarificationRequests", ccr);
        params.put("claimProcessingAborted", cpa);

        List<Command<?>> commands = new ArrayList<>();
        KieServices kieServices = KieServices.Factory.get();
        KieCommands commandFactory = kieServices.getCommands();
        commands.add(commandFactory.newStartProcess("claims.ClaimRuleFlow", params));
        commands.add(commandFactory.newFireAllRules());
        commands.add(commandFactory.newGetObjects("claimResults"));

        BatchExecutionCommand batchExecutionCommand = commandFactory.newBatchExecution(commands);

        ServiceResponse<ExecutionResults> response = client.executeCommandsWithResults(CONTAINER, batchExecutionCommand);
        ExecutionResults results = response.getResult();
        System.out.println(results.getValue("claimResults"));
    }
}
